require 'test_helper'

class FredFlinstone
  attr_reader :weight, :age

  def initialize(weight, age)
    @weight = weight
    @age    = age
  end

  def next_years_weight
    increase = weight / age
    increase * (age + 1)
  end
end

BamBam = Recite.copy(FredFlinstone) do
  def age
    5
  end
end


class TestMimic < Minitest::Test
  def setup
    @copy     = BamBam.new weight: 100
    @instance = Recite.instance(FredFlinstone, weight: 60) do
      def age
        8
      end
    end
  end

  def test_reports_as_the_correct_is_a?
    assert @copy.is_a? FredFlinstone
    assert @instance.is_a? FredFlinstone
  end

  def test_reports_the_correct_kind_of?
    assert @copy.kind_of? FredFlinstone
    assert @instance.kind_of? FredFlinstone
  end

  def test_reports_the_correct_instance_of?
    assert @copy.instance_of? FredFlinstone
    assert @instance.instance_of? FredFlinstone
  end

  def test_uses_overridden_block_method
    assert_equal 5, @copy.age
    assert_equal 8, @instance.age
  end

  def test_uses_overridden_constructor_method
    assert_equal 100, @copy.weight
    assert_equal 60, @instance.weight
  end

  def test_delegates_down_to_original_method
    assert_equal 120, @copy.next_years_weight
    assert_equal 63, @instance.next_years_weight
  end
end
